@@ -17,6 +17,8 @@ echo -n "creating test environment..."
 $PGPOOL_SETUP -m s -n 2 || exit 1
 echo "done."
 
+echo "search_primary_node_timeout = 5" >> etc/pgpool.conf
+
 source ./bashrc.ports
 
 export PGPORT=$PGPOOL_PORT
@@ -31,7 +33,7 @@ sh pgpool.sh
 
 while true
 do
-	grep 'failover done. shutdown host /tmp(11003)' log/pgpool.log > /dev/null 2>&1
+	grep "failover done. shutdown host $PGSOCKET_DIR(11003)" log/pgpool.log > /dev/null 2>&1
 	if [ $? = 0 ];then
 		break;
 	fi
